/* -----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: RTX example program
 * ----------------------------------------------------------------------------- */

#include <stdio.h>

#include "cmsis_os2.h"                  // ARM::CMSIS:RTOS2:Keil RTX5
#include "Board_LED.h"                  // ::Board Support:LED
#include "Board_Buttons.h"              // ::Board Support:Buttons

#include "RTE_Components.h"             // Component selection
#include CMSIS_device_header

volatile int32_t delay_val = 500;

osThreadId_t tid_thrLED;                // Thread id of thread: LED
osThreadId_t tid_thrButton;             // Thread id of thread: Button

/*
 * thrLED: blink LED and check button state
 */
void thrLED(void *argument) {
  uint32_t led_max    = LED_GetCount();
  uint32_t led_num    = 0;

  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    LED_On(led_num);                                           // Turn specified LED on
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    LED_Off(led_num);                                          // Turn specified LED off

    led_num++;                                                 // Change LED number
    if (led_num >= led_max) {
      led_num = 0;                                             // Restart with first LED
    }
  }

}

/*
 * thrButton: blink LED and check button state
 */
void thrButton(void *argument) {
  uint32_t button_msk = (1U << Buttons_GetCount()) - 1;

  for (;;) {
    osDelay(delay_val);
    while (Buttons_GetState() & (button_msk));                 // Wait while holding USER button
    osThreadFlagsSet(tid_thrLED, 0x0001);
  }

}


/*----------------------------------------------------------------------------
 * main: blink LED and check button state
 *----------------------------------------------------------------------------*/
int main (void) {

  osKernelInitialize ();                                       // initialize CMSIS-RTOS

  SystemCoreClockUpdate();

  LED_Initialize();                                            // initialize LEDs
  Buttons_Initialize();                                        // initialize Buttons

  osKernelInitialize ();                                       // initialize CMSIS-RTOS

  tid_thrLED    = osThreadNew (thrLED,   NULL, NULL);          // create LED thread
  tid_thrButton = osThreadNew (thrButton, NULL, NULL);         // create Button thread

  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                                           // start thread execution
  }

  while(1);
}
